<?php

declare(strict_types=1);

namespace Pinpoint\RakutenTracking\ViewModel;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\Checkout\Model\Session;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\View\Element\Block\ArgumentInterface;
use Pinpoint\RakutenTracking\Helper\Data;

class CheckoutTracking implements ArgumentInterface
{
    /**
     * @var Session
     */
    private $checkoutSession;
    /**
     * @var Json
     */
    private $serializer;
    /**
     * @var CollectionFactory
     */
    private $categoryCollectionFactory;
    /**
     * @var Data
     */
    private $dataHelper;

    /**
     * @param Session           $checkoutSession
     * @param Json              $serializer
     * @param CollectionFactory $categoryCollectionFactory
     * @param Data              $dataHelper
     */
    public function __construct(
        Session $checkoutSession,
        Json $serializer,
        CollectionFactory $categoryCollectionFactory,
        Data $dataHelper
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->serializer = $serializer;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->dataHelper = $dataHelper;
    }

    /**
     * Retrieves json formatted data
     *
     * @return string
     */
    public function getJson(): string
    {
        return $this->serializer->serialize($this->getTrackingData());
    }

    /**
     * Gathers all needed tracking data from the order
     *
     * @return array
     */
    protected function getTrackingData(): array
    {
        $order = $this->checkoutSession->getLastRealOrder();

        if (!$order->getId()) {
            return [];
        }

        $lineItems = [];

        foreach ($order->getAllVisibleItems() as $orderItem) {
            /** @var Item $orderItem */
            $lineItemData = [
                'quantity' => $orderItem->getQtyOrdered(),
                'unitPrice' => $orderItem->getPriceInclTax(),
                'unitPriceLessTax' => $orderItem->getPrice(),
                'SKU' => $orderItem->getSku(),
                'productName' => $orderItem->getName()
            ];

            $optionalData = $this->getProductOptionalData($orderItem->getProduct());

            if (!empty($optionalData)) {
                $lineItemData["optionalData"] = $optionalData;
            }

            $lineItems[] = $lineItemData;
        }

        $affiliateConfig = [
            'ranMID' => $this->dataHelper->getMidNumber(),
            'discountType' => 'item',
            'includeStatus' => 'false',
            'taxRate' => 20,
            'removeTaxFromDiscount' => true
        ];
        $trackingData = [
            'affiliateConfig' => $affiliateConfig,
            'orderid' => $order->getIncrementId(),
            'currency' => $order->getOrderCurrencyCode(),
            'customerStatus' => 'New',
            'conversionType' => 'Sale',
            'discountCode' => $order->getCouponCode(),
            'discountAmount' => $order->getDiscountAmount(),
            'taxAmount' => $order->getTaxAmount(),
            'lineitems' => $lineItems
        ];
        $customerId = $order->getCustomerId();
        if(!empty($customerId)){
            $trackingData['customerID'] = $customerId;
        }
        return $trackingData;
    }

    /**
     * Prepares product additional data
     *
     * @param ProductInterface $product
     * @return array
     */
    private function getProductOptionalData(ProductInterface $product): array
    {
        $optionalData = [];
        $categoryNames = [];
        $categories = $this->fetchProductCategories($product->getCategoryIds());

        foreach ($categories as $category) {
            $categoryNames[] = $category->getName();
        }

        if (!empty($categories)) {
            $optionalData['cat'] = implode('>', $categoryNames);
        }
        if (!empty($product->getBrand())) {
            $optionalData['brand'] = $product->getBrand();
        }

        return $optionalData;
    }

    /**
     * Fetches category names
     *
     * @param array|null $ids
     * @return array
     */
    private function fetchProductCategories(?array $ids): array
    {
        if (empty($ids)) {
            return [];
        }

        $categoryCollection = $this->categoryCollectionFactory->create();
        $categoryCollection->addNameToResult()
            ->addFieldToFilter('entity_id', ['in' => $ids])
            ->getSelect()
            ->limit(5);

        return $categoryCollection->getItems();
    }
}
